@@ -6,16 +6,18 @@
 # Author: jianglin
 # Email: xiyang0807@gmail.com
 # Created: 2017-07-12 21:21:00 (CST)
-# Last Update:星期三 2017-7-26 11:9:17 (CST)
+# Last Update:星期五 2017-9-15 11:0:54 (CST)
 #          By:
 # Description:
 # **************************************************************************
 import re
 from time import time
+from collections import OrderedDict
 
 
 class Regex(object):
-    newline = re.compile(r'^$')
+    blankline = re.compile(r'^$')
+    newline = re.compile(r'\\$')
     heading = re.compile(r'^(?P<level>\*+)\s+(?P<title>.+)$')
     comment = re.compile(r'^(\s*)#(.*)$')
     bold = re.compile(r'( |^)\*(?P<text>[\S]+)\*')
@@ -25,14 +27,23 @@ class Regex(object):
     code = re.compile(r'( |^)=(?P<text>[\S]+)=')
     delete = re.compile(r'( |^)\+(?P<text>[\S]+)\+')
     verbatim = re.compile(r'( |^)~(?P<text>[\S]+)~')
-    image = re.compile(r'\[\[(?P<src>.+?)\](?:\[(?P<alt>.+?)\])?\]')
+    image = re.compile(r'\[\[(?P<text>.+?)[.](jpg | png | gif)\]\]')
     link = re.compile(r'\[\[(?P<href>https?://.+?)\](?:\[(?P<text>.+?)\])?\]')
+    origin_link = re.compile(
+        r'(http|ftp|https):\/\/[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#]*[\w\-\@?^=%&amp;/~\+#])?'
+    )
     fn = re.compile(r'\[fn:(?P<text>.+?)\]')
     hr = re.compile(r'^\s*\-{5,}\s*')
 
     begin_example = re.compile(r'\s*#\+BEGIN_EXAMPLE$')
     end_example = re.compile(r'\s*#\+END_EXAMPLE$')
 
+    begin_verse = re.compile(r'\s*#\+BEGIN_VERSE$')
+    end_verse = re.compile(r'\s*#\+END_VERSE$')
+
+    begin_center = re.compile(r'\s*#\+BEGIN_CENTER$')
+    end_center = re.compile(r'\s*#\+END_CENTER$')
+
     begin_quote = re.compile(r'\s*#\+BEGIN_QUOTE$')
     end_quote = re.compile(r'\s*#\+END_QUOTE$')
 
@@ -57,18 +68,17 @@ class NotBeginError(Exception):
 
 class InlineElement(object):
     label = '{text}'
-    regex = None
 
-    def __init__(self, text):
+    def __init__(self, text, regex):
         self.text = text
+        self.regex = regex
         self.children = []
 
     def to_html(self):
-        return self.parse(self.text)
+        def _match(match):
+            return self.label.format(text=match.group('text'))
 
-    def parse(self, text):
-        return self.regex.sub(
-            lambda match: self.label.format(text=match.group('text')), text)
+        return self.regex.sub(_match, self.text)
 
     def __str__(self):
         return '{}({})'.format(self.__class__.__name__, self.text.strip())
@@ -79,73 +89,62 @@ class Fn(InlineElement):
     <sup><a id="fnr.1" class="footref" href="#fn.1">1</a></sup>
     '''
     label = '<sup><a id="fnr:{text}" class="footref" href="#fn.{text}">{text}</a></sup>'
-    regex = Regex.fn
 
 
 class Underlined(InlineElement):
     label = '<span style="text-decoration:underline">{text}</span>'
-    regex = Regex.underlined
 
 
 class Bold(InlineElement):
     label = '<b>{text}</b>'
-    regex = Regex.bold
 
 
 class Italic(InlineElement):
     label = '<i>{text}</i>'
-    regex = Regex.italic
 
 
 class Code(InlineElement):
     label = '<code>{text}</code>'
-    regex = Regex.code
 
 
 class Delete(InlineElement):
     label = '<del>{text}</del>'
-    regex = Regex.delete
 
 
 class Verbatim(InlineElement):
     label = '<code>{text}</code>'
-    regex = Regex.verbatim
+
+
+class NewLine(InlineElement):
+    def to_html(self):
+        return self.regex.sub('<br/>', self.text)
 
 
 class Image(InlineElement):
-    label = '<img alt="{alt}" src="{src}"/>'
-    label1 = '<img src="{src}"/>'
-    regex = Regex.image
-
-    def parse(self, text):
-        # return self.regex.sub(
-        #     lambda match: self.label.format(text=match.group('text'))
-        #     if not match.group('alt') else
-        #     self.label.format(src=match.group('src'),
-        #                       alt=match.group('alt')), text)
-        for t in self.regex.finditer(text):
-            if not t.group('alt'):
-                string = self.label1.format(src=t.group('src'))
-            else:
-                string = self.label.format(
-                    src=t.group('src'), alt=t.group('alt'))
-            text = self.regex.sub(string, text, 1)
-        return text
+    label = '<img src="{text}"/>'
 
 
 class Link(InlineElement):
     label = '<a href="{href}">{text}</a>'
-    regex = Regex.link
 
-    def parse(self, text):
-        return self.regex.sub(
-            lambda match: self.label.format(href=match.group('href'),
-                                       text=match.group('text')), text)
-        # for t in self.regex.finditer(text):
-        #     string = self.label.format(
-        #         href=t.group('href'), text=t.group('text'))
-        #     text = self.regex.sub(string, text, 1)
-        # return text
+    def to_html(self):
+        def _match(match):
+            if not match.group('text'):
+                return Image.label.format(text=match.group('href'))
+            return self.label.format(
+                href=match.group('href'), text=match.group('text'))
+
+        return self.regex.sub(_match, self.text)
+
+
+class OriginLink(InlineElement):
+    label = '<a href="{0}">{0}</a>'
+
+    def to_html(self):
+        def _match(match):
+            return self.label.format(match.group())
+
+        return self.regex.sub(_match, self.text)
 
 
 class Heading(InlineElement):
@@ -159,20 +158,37 @@ class Heading(InlineElement):
         self._toc = toc
         self.children = []
 
-    def parse(self, text):
-        m = Regex.heading.match(text)
+    def to_html(self):
+        m = Regex.heading.match(self.text)
         level = len(m.group('level')) + self.offset
         title = m.group('title')
         text = self.label.format(level=level, title=title)
         if self._toc:
-            tid = 'org-{}'.format(int(time() * 10000))
+            tid = self.heading_id(text)
             text = self.label1.format(level=level, tid=tid, title=title)
             self.toc = '{}- <a href="#{}">{}</a>'.format(' ' * level, tid,
                                                          title)
         return text
 
+    def heading_id(self, text):
+        return 'org-{}'.format(int(time() * 10000))
+
 
 class Text(InlineElement):
+    regex = OrderedDict({
+        'comment': Regex.comment,
+        'newline': Regex.newline,
+        'italic': Regex.italic,
+        'bold': Regex.bold,
+        'underlined': Regex.underlined,
+        'code': Regex.code,
+        'delete': Regex.delete,
+        'verbatim': Regex.verbatim,
+        'fn': Regex.fn,
+        'link': Regex.link,
+        'image': Regex.image,
+    })
+
     def __init__(self, text, no_parse=False):
         self.text = text
         self.no_parse = no_parse
@@ -180,32 +196,46 @@ class Text(InlineElement):
     def parse(self, text):
         if not isinstance(text, str):
             text = text.to_html()
-        if Regex.comment.search(text) or self.no_parse:
+        if self.no_parse:
             return text
-        elif Regex.italic.search(text):
-            return self.parse(Italic(text))
-        elif Regex.bold.search(text):
-            return self.parse(Bold(text))
-        elif Regex.underlined.search(text):
-            return self.parse(Underlined(text))
-        elif Regex.code.search(text):
-            return self.parse(Code(text))
-        elif Regex.delete.search(text):
-            return self.parse(Delete(text))
-        elif Regex.verbatim.search(text):
-            return self.parse(Verbatim(text))
-        elif Regex.fn.search(text):
-            return self.parse(Fn(text))
-        else:
-            return self.parse_other(text)
-
-    def parse_other(self, text):
-        if Regex.link.search(text):
-            text = Link(text).to_html()
-        elif Regex.image.search(text):
-            text = Image(text).to_html()
+        for parse, regex in self.regex.items():
+            if regex.search(text):
+                return getattr(self, 'parse_' + parse)(text, regex)
         return text
 
+    def parse_comment(self, text, regex):
+        return text
+
+    def parse_newline(self, text, regex):
+        return self.parse(NewLine(text, regex))
+
+    def parse_italic(self, text, regex):
+        return self.parse(Italic(text, regex))
+
+    def parse_bold(self, text, regex):
+        return self.parse(Bold(text, regex))
+
+    def parse_underlined(self, text, regex):
+        return self.parse(Underlined(text, regex))
+
+    def parse_code(self, text, regex):
+        return self.parse(Code(text, regex))
+
+    def parse_delete(self, text, regex):
+        return self.parse(Delete(text, regex))
+
+    def parse_verbatim(self, text, regex):
+        return self.parse(Verbatim(text, regex))
+
+    def parse_fn(self, text, regex):
+        return self.parse(Fn(text, regex))
+
+    def parse_link(self, text, regex):
+        return self.parse(Link(text, regex))
+
+    def parse_image(self, text, regex):
+        return self.parse(Image(text, regex))
+
     def to_html(self):
         return self.parse(self.text)
 
@@ -233,28 +263,10 @@ class Element(object):
         return self.__class__.__name__ + '(' + ','.join(str_children) + ')'
 
 
-class Example(Element):
-    label = '\n<pre class="example">\n{text}\n</pre>\n'
-
-    def __init__(self, parent):
-        self.parent = parent
-        self.flag = False
-        self.children = []
-
-    def append(self, child):
-        if self.children or child:
-            if isinstance(child, str):
-                child = Text(child, True)
-            self.children.append(child)
-
-    def end(self, text):
-        return Regex.end_example.match(text)
-
-
 class Src(Element):
     label = '<pre class="{lang}">\n{text}\n</pre>'
 
-    def __init__(self, parent, lang='python'):
+    def __init__(self, parent, lang='example'):
         self.parent = parent
         self.lang = lang
         self.flag = False
@@ -274,6 +286,42 @@ class Src(Element):
         return Regex.end_src.match(text)
 
 
+class Example(Src):
+    def end(self, text):
+        return Regex.end_example.match(text)
+
+
+class Verse(Element):
+    label = '<p class="org-verse">\n{text}\n</p>'
+
+    def __init__(self, parent):
+        self.parent = parent
+        self.flag = False
+        self.children = []
+
+    def to_html(self):
+        text = '<br/>\n'.join([child.to_html() for child in self.children])
+        return self.label.format(text=text)
+
+    def end(self, text):
+        return Regex.end_verse.match(text)
+
+
+class Center(Element):
+    label = '<div class="org-center">\n{text}\n</div>'
+
+    def __init__(self, parent):
+        self.parent = parent
+        self.flag = False
+        self.children = [Org('', parse=False)]
+
+    def append(self, child):
+        self.children[0].parse(child)
+
+    def end(self, text):
+        return Regex.end_center.match(text)
+
+
 class BlockQuote(Element):
     label = '<blockquote>\n{text}\n</blockquote>'
 
@@ -424,7 +472,37 @@ class Paragraph(Element):
     label = '<p>{text}</p>'
 
 
+class Re(object):
+    def match(self, text):
+        pass
+
+
 class Org(object):
+    class _end:
+        def __init__(self, _self):
+            self._self = _self
+
+        def match(self, text):
+            _self = self._self
+            return hasattr(_self.current, 'flag') and _self.current.flag
+
+    regex = OrderedDict({
+        'end': _end,
+        'heading': Regex.heading,
+        'unorderlist': Regex.unorder_list,
+        'orderlist': Regex.order_list,
+        'table': Regex.table,
+        'quote': Regex.begin_quote,
+        'verse': Regex.begin_verse,
+        'center': Regex.begin_center,
+        'example': Regex.begin_example,
+        'src': Regex.begin_src,
+        'hr': Regex.hr,
+        'attr': Regex.attr,
+        'blankline': Regex.blankline,
+    })
+    del _end
+
     def __init__(self, text, offset=0, toc=False, parse=True):
         self.text = text
         self.children = []
@@ -434,35 +512,20 @@ class Org(object):
         self.toc = Toc(self)
         self.toc.flag = toc
         if parse:
-            for line in text.splitlines():
-                self.parse(line.rstrip())
+            self._parse(text)
+
+    def _parse(self, text):
+        for line in text.splitlines():
+            self.parse(line.rstrip())
 
     def parse(self, text):
-        if hasattr(self.current, 'flag') and self.current.flag:
-            self.parse_end(text)
-        elif Regex.heading.match(text):
-            self.parse_heading(text)
-        elif Regex.unorder_list.match(text):
-            self.parse_unorderlist(text)
-        elif Regex.order_list.match(text):
-            self.parse_orderlist(text)
-        elif Regex.table.match(text):
-            self.parse_table(text)
-        elif Regex.begin_quote.match(text):
-            self.parse_quote(text)
-        elif Regex.begin_example.match(text):
-            self.parse_example(text)
-        elif Regex.begin_src.match(text):
-            self.parse_src(text)
-        elif Regex.hr.match(text):
-            self.parse_hr(text)
-        elif Regex.attr.match(text):
-            pass
-        elif not text.strip():
-            while isinstance(self.current, Paragraph):
-                self.current = self.current.parent
-            self.children.append(Text(''))
-        elif isinstance(self.current, Paragraph):
+        for parse, regex in self.regex.items():
+            if callable(regex):
+                regex = regex(self)
+            if regex.match(text):
+                getattr(self, 'parse_' + parse)(text)
+                return
+        if isinstance(self.current, Paragraph):
             self.current.append(text.strip())
         else:
             while isinstance(self.current, Paragraph):
@@ -496,6 +559,11 @@ class Org(object):
             if isinstance(e, (UnorderList, OrderList)):
                 self.parse(text)
 
+    def parse_blankline(self, text):
+        while isinstance(self.current, Paragraph):
+            self.current = self.current.parent
+        self.children.append(Text(''))
+
     def parse_heading(self, text):
         element = Heading(text, self.offset, self.toc.flag)
         self.toc.append(element)
@@ -541,6 +609,17 @@ class Org(object):
         element = BlockQuote(self.current)
         self.begin_init(element)
 
+    def parse_center(self, text):
+        element = Center(self.current)
+        self.begin_init(element)
+
+    def parse_verse(self, text):
+        element = Verse(self.current)
+        self.begin_init(element)
+
+    def parse_attr(self, text):
+        pass
+
     def append(self, child):
         if isinstance(child, str):
             child = Text(child)
